@rem = ' This is a perl script for NT and UNIX.
@echo off
goto endofperl
@rem ';
# -*- tab-width: 4 -*-
#---------------------------------------------------------------------------+
# Licensed Materials - Property of IBM
# Rational ClearCase
# Copyright IBM Corp. 1998, 2008. All Rights Reserved
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract
# with IBM Corp.
#---------------------------------------------------------------------------+

###########################################################################
# syncmgr_receive.bat
# usage: syncmgr_receive.bat - run with "-help" for detailed usage.
#
# This script is intended to be run periodically by the ClearCase
# Job Scheduler to import ClearCase MultiSite packets that have been received
# from other replicas to replicas that are located on the local system.
#
# This script may also be executed interactively from the command line to
# force an import to occur immediately, either for a particular packet, for
# a particular VOB, or for a storage class.
###########################################################################

use strict;
use Getopt::Long;
use File::Basename;

BEGIN {
    if($ =~ /^(MS(DOS|Win32)|Windows_NT)/i) {
        require Win32::TieRegistry;
    }
}
use ClearCase::CurrentVersion;

sub dbgprint;  # predeclare, routine is in require file
sub qprint;    # predeclare
sub print2;    # predeclare

my $thisscript = $0;
if ($thisscript =~ m/(.*)(\.bat)$/) {
    $thisscript = $1;
}
$CFG::including_script = $thisscript;

my $pkg_name = 'syncmgr_receive';
$CFG::verbose = ($ENV{TRACE_SUBSYS} && ($ENV{TRACE_SUBSYS} =~ m/$pkg_name/));
# force debugging msgs by uncommenting the following and setting to 1.....
#$CFG::verbose = 1;

$CFG::quietmode = 0;

# if 'verbose' mode is on, force the debug msgs to also print to STDERR by
# uncommenting this line (forces debug msgs to appear in the log file,
# useful if you're having problems with a receipt handler).
#$CFG::stderr_msgs = 1;



# set script version to test for mismatch against the syncmgr_common.pl version.
$CFG::MS_script_version = "7.0.0.0_syncmgr";

# figure out the machine's architecture
($CFG::NT, $CFG::UNIX) = GetArchitecture();

# CLEARCASEHOME e.v. should always be set on NT when called by ccperlpp.pl,
# so exit if it's not set. On UNIX, ClearCase::CurrentVersion::SetProductHomeEV()
# assumes /opt/rational/clearcase if user hasn't explicitly set CLEARCASEHOME 
# (or deprecated ATRIAHOME).

if ($CFG::UNIX) {
    my $producthome = &ClearCase::CurrentVersion::SetProductHomeEV;
    chdir $producthome;
    push(@INC, "$producthome/config/scheduler/tasks");
} else {
    my $producthome = $ENV{CLEARCASEHOME} or 
                      die "CLEARCASEHOME env. variable not set\n";
    chdir $producthome;
    push(@INC, "$producthome\\config\\scheduler\\tasks");
    $thisscript = $pkg_name;
}

require 'syncmgr_common.pl';
require 5.8.6;

if (!$CFG::MScommon_version) {
    $CFG::MScommon_version = "(unknown older version)";
}

# check for mismatched versions - syncmgr_common.pl should set 
# $CFG::MScommon_version; if not the same or not initialized,
# indicates that an incompatible older/newer version is
# being included improperly.

if ($CFG::MScommon_version ne $CFG::MS_script_version) {
    die "Version mismatch between require file 'syncmgr_common.pl' " .
        "(version: ${CFG::MScommon_version}) and \n" .
        "the MultiSite import script which includes it, 'syncmgr_receive.bat'" .
        " (version: ${CFG::MS_script_version}).\n" . 
        "Ensure that old versions of the script/require file are not in use.\n";
}

$CFG::failcount = 0;

$| = 1 if $CFG::verbose;

dbgprint "running in directory: ", DoCmd($CFG::workingdir_cmd);
my $arch = $CFG::NT ? "Windows" : "UNIX";
dbgprint "running on architecture $arch\n";

###########################################################################
# main
#

my $usage = "$thisscript [-help] [-vob vob-patt] [-workdir workdir]\n" .
    "    [-logdir logdir] [-sclass sclass] [-lockwait minutes]\n" .
    "    [-blocking_duration minutes | -nblocking | -wait_forever]\n" .
    "    [-[no]trace] [-origin orig-host] [-quiet mode] [-view viewtag]\n" .
    "    [-datafile {file | dir}] [-actual shiporder]\n\n" .
    "    The shortest umambiguous abbreviation for switches may be used.\n" .
    "    More help is available by using the '-help' command-line switch.\n";


my $helpmsg = <<ENDHELP;
=========================================================================
"syncmgr_receive" is a general importer for incoming CC MultiSite packets. It 
may be used interactively from the command line, run via periodic scheduler 
(ClearCase job scheduler), or it will also function as an automatic receipt 
handler triggered by the arrival of incoming packets.

NOTE: this script is intended to be used if you are using "enhanced" mode 
MultiSite with the synchronization manager (syncmgr). If you use "standard"
mode MultiSite, use the "sync_receive" script instead.

A logfile is created during execution, but is normally deleted if the script 
executes successfully, and is kept if a failure is detected.

Setting the environment variable TRACE_SUBSYS to "syncmgr_receive" will turn
on all debugging print statements (verbose mode). Using the "-trace" command
line switch will print a less-verbose set of messages. Using either tracing
method will retain the logfile even if the script is successful.

$usage
Command line options:

$CFG::help_parm_help
$CFG::logdir_parm_help
$CFG::workdir_parm_help
$CFG::quiet_parm_help
$CFG::trace_parm_help
$CFG::view_parm_help
    -lockwait minutes   The amount of time (in minutes) to wait on VOB 
			locks that block VOB writes. Minutes must be > 0.
			(Default: no waiting).
			NOTE: If VOB locked and lockwait NOT set, import
			will exit with error.

                        The CLEARCASE_VOBLOCKWAIT e.v. is ignored by this 
                        script, use this parameter instead.

    -vob pattern        Specify VOB to apply imports. This parameter is a
                        pattern-match: you may either specify the whole VOB
                        name (only packets with contents for that VOB will
                        be applied); or a pattern which could match multiple 
                        VOB names. Examples:
                        -vob /vobs/dev/george1 (apply packets to 
                            /vobs/dev/george1 only)
                        -vob george (applies to any 'george' VOBs such as 
                            /vobs/dev/george1, /vobs/dev/george2,
                            /vobs/george_pvt/myvob, etc.).
                        (Default: all VOBs in any packets found).

    -sclass class       Storage class to use. (Default: all classes). This
                        parameter is also used when the script is invoked as
                        a receipt handler. Using this parameter restricts the
                        script to finding packets in the incoming bay(s) of
                        this storage class. If the storage class does not
                        have its own bay(s) set up, the default storage 
                        bays are used.

    -blocking_duration minutes
                        The amount of time (> 0, in minutes) that the script
                        should wait for an import before continuing.

                        This parameter is mutually exclusive with "-nblocking"
                        or "-wait_forever".

                        If the script continues, the job will still be
                        queued to the sync management system.

                        See multitool "-blocking_duration" parameter for 
                        more info.
               
    -nblocking          Multitool will queue jobs to the sync management
                        system but will not wait for completion before
                        allowing the script to continue.

                        See multitool "-nblocking" parameter for more info.
               
    -wait_forever       Multitool will not return control to the script until
                        it has completed its request, except when an import job
                        is waiting for changes in another packet.

                        See multitool "-wait_forever" parameter for more info.
               
The following parameters are passed in automatically when the script has been
invoked as a receipt handler. The -d (datafile) parameter may be useful from
the command line if import of a particular packet or bay is desired.

When invoked as a receipt handler, note that only the '-o' parameter is
guaranteed to be passed, along with one of '-a' or '-d' (or possibly both).
The '-s' parameter is only passed if a specific storage-class has been
specified.

    -origin hostname    Originating host. Only function in the script is to
                        force keeping the log file (so that all receipt
                        handler logs are kept). Specifying -o also forces the
                        script to run in "-quiet 1" mode to avoid writing 
                        excessive informational messages to the receipt
                        handler logfiles.

    -datafile file | dir 
                        Data packet\'s full pathname. In addition to being
                        used in receipt-handler mode, this may be specified
                        interactively to import only a specific packet. This
                        may be also be used with the -vob parameter to import
                        to specific VOBs only (if the packet has multiple
                        targets).

                        If a directory path is specified, the
                        script will attempt to import any packets found in
                        that directory.

    -sclass storage-class 
                        Storage class (see -sclass above).

    -actual shipping-order 
                        Actual pathname of the new shipping order if this
                        packet is destined for another host (i.e. this
                        system is a 'hop host' for the packet). Specifying
                        a shipping order file in this parameter will cause
                        the shipping server to send this shipping order to
                        its next host destination.

Previous versions of MultiSite used a script configuration file, located at 
'${CFG::script_configfile}' 
to change the behavior of the script. This configuration file is no 
longer required, and will be ignored if present.
=========================================================================
ENDHELP

# get command line options figured out...
dbgprint "ARGV: " . join(' ',@ARGV) . "\n";

# delete local voblockwait EV - user must specify "-lockwait"
RemoveLockwaitEV();

my $vob_pattern = '';
my $logdir = '';
my $help = 0;
$CFG::traceflg = 0;
my $lockwait = 0;
my $origin_host = '';
my $sclass = '';
my $actual_shiporder = '';
my $datafile = '';
my $keeplogfile = 0;
my $view = '';
my $blocking_dur = 0;
my $wait_forever = 0;
my $nblocking = 0;
my $workdir = '';
my @SAVE_ARGV = @ARGV;
my $multitool_cmd = "$CFG::MultiTool syncreplica -import ";

my $st = GetOptions("logdir=s" => \$logdir,
                 "datafile=s" => \$datafile,
                 "origin=s"  => \$origin_host,
                 "vob=s" => \$vob_pattern,
                 "help!" => \$help,
                 "trace!" => \$CFG::traceflg,
                 "lockwait=i" => \$lockwait,
                 "view=s" => \$view,
                 "blocking_duration=i" => \$blocking_dur,
                 "wait_forever!" => \$wait_forever,
                 "nblocking!" => \$nblocking,
                 "workdir=s" => \$workdir,
                 "quiet=i" => \$CFG::quietmode,
                 "actual=s" => \$actual_shiporder,
                 "sclass=s" => \$sclass);

if (!$st) {
    qprint "ERROR: error getting command line options.\nUsage: $usage\n";
    exit 1;
}

if ($help) {
    qprint "$helpmsg\n";
    exit 0;
}

if ($view) {
    $multitool_cmd = $multitool_cmd . "-view $view ";
}

if ($workdir) {
    $multitool_cmd = $multitool_cmd . "-workdir $workdir ";
}    

if ($lockwait) {
    $multitool_cmd = $multitool_cmd . "-lockwait $lockwait ";
}

# are we a receipt handler? set flag and quiet mode.
if ($origin_host && -f $datafile) {    # are we a receipt handler? set flag.
    $CFG::receipt_handler = 1;
    $CFG::quietmode = 1;
} else {
    $CFG::receipt_handler = 0;
}

# if "-blocking_duration" is not specified, OR if it seems we're a 
# receipt handler, we will not block; just queue and exit.

$multitool_cmd .= MakeBlockingExpr($nblocking, $blocking_dur, $wait_forever);

my $options_msg = "options:\n    vob_pattern: '$vob_pattern'\n" . 
    "    datafile (or dir): '$datafile'\n" . 
    "    originating host: '$origin_host'\n    workdir:    '$workdir'\n" .
    "    logdir: '$logdir'\n    storage-class: '$sclass'\n" .
    "    lockwait: '$lockwait minutes'\n" .
    "    blocking_duration: '$blocking_dur minutes'\n" .
    "    nblocking: '$nblocking'\n" .
    "    wait_forever: '$wait_forever'\n" .
    "    actual shipping order: '$actual_shiporder'\n" .
    "    view: '$view'\n";

dbgprint $options_msg;

#
# Validate that the log directory actually exists...
#
$logdir = ValidateLogDir($logdir);

#
# Redirect STDERR to a uniquely named log file
#
my $logname = RedirectSTDERRtoLog ($logdir, 'recv');

my $job_id_tmpfile = $logname . "_jfile";
$multitool_cmd = $multitool_cmd . "-jfile \"$job_id_tmpfile\" ";

print STDERR "=========================================================\n" .
    "Starting MultiSite import from script '$thisscript' at " .
    MakeTimestamp() . "\n";

print STDERR "Actual arguments: " . join(' ',@SAVE_ARGV) . "\n";

print STDERR $options_msg;

my ($cmd, $res);

my @import_pkts = ();

if ($datafile) {
    # If it's a Windows platform, make sure we have the unabbreviated name
    if ($CFG::NT) {
	$datafile = Win32::GetLongPathName($datafile);
    }

    if (-f $datafile) {
        # single file; make list from it.
        @import_pkts = ($datafile);
    } elsif (-d $datafile) {
        # directory - get all 'regular "-f" files
        my @dfiles = GetDirFiles($datafile);
        dbgprint "from GetDirFiles() " . join(" ",@dfiles) . "\n";
        unshift (@import_pkts, @dfiles);
    }
} elsif (!$origin_host) {
    # we are NOT a receipt handler if $origin_host is NULL, and no file
    # or directory was specified. So, look thru the storage bays....
    # locate storage bays for the storage class (all bays if $sclass is null)
    # and save all the files found in the bay. Some may NOT be pkts, that
    # will be determined later.

    my (@bays) = GetAllBays($sclass);
    # scan thru the shipping bays. Need to add the "incoming" subdirectory
    foreach my $dir (@bays) {
        my $baydir = $dir . $CFG::FS . "incoming";
        my @dfiles = GetDirFiles($baydir);
        dbgprint "Shipping bay directory: $baydir\n";
        unshift (@import_pkts, @dfiles);
    }
}

dbgprint "import_pkts: " . join("\n", @import_pkts) . "\n";

##############
# begin import
##############

if ((scalar (@import_pkts) == 0)) {
    print2 "INFO: no valid packets were found to import.\n";
}

# attempt to decompress/import each file that has been collected, ignoring any
# non-packets.
foreach my $pkt (@import_pkts) {
    ImportFile($pkt);
}

# was a shipping order specified on the cmd line? If so, try to launch it 
if ($actual_shiporder) {
    if (-e $actual_shiporder) {
	my $cmd = qq/$CFG::shipping_server "$actual_shiporder"/;
	$res = Do_System($cmd, __FILE__, __LINE__);
	if ($res == 0) {
	    if ($CFG::NT) {
		$cmd = "$CFG::imsglog sync_import evtlog";
		$res = Do_System($cmd, __FILE__, __LINE__);
	    }
	} else {
	    #
	    # Shipping server failed.  Update event log with pointer to log.
	    #
            $CFG::failcount++;
	    if ($CFG::NT) {
		$cmd = "$CFG::imsglog sync_import evtlog \"$logname\"";
		$res = Do_System($cmd, __FILE__, __LINE__);
	    }
            my $msg = "ERROR: attempt to forward shipping order " .
                "$actual_shiporder failed.\n";
	    print2 $msg;
	    dbgprint $msg ."See logfile in \"$logname\"\n";
        }
    } else {
        # can't find the shipping order specified. It may have already 
        # been sent if the scheduler or user fired off a 
        # 'shipping_order -poll'. Print a warning, but don't flag as an error.

        print2 "WARNING: shipping order $actual_shiporder not found " .
            "(perhaps previously sent?).\n";
    }
}

if ($CFG::traceflg || $CFG::verbose || $CFG::failcount) {
    $keeplogfile = 1;
}

if ($CFG::failcount) {
    print2 qq/\nERROR: $CFG::failcount import error(s) detected!\nLog is "$logname"\n/;
} else {
    print2 "\nSUCCESSFUL COMPLETION of MultiSite import.\n";
    if ($keeplogfile) {
        qprint "See logfile at: $logname.\n";
    }
}
close(STDOUT);
close(STDERR);
if (!$keeplogfile) {
    unlink $logname;
}
exit $CFG::failcount;

sub GetJobIds {
    # Parse a file and return job_ids

    my $job_file = shift;
    if (-e $job_file) {
        open (JF, "< $job_file")
            or die "Could not open job id file $job_file for reading: $!\n";
        
        my @job_list = ();
        my %seen = ();
        while (<JF>) {
            chomp;
            push (@job_list, $_) unless $seen{$_}++;
        }
        close JF;
        return @job_list;
    } else {
        return ();
    }
}


sub GetJobState {
    # Parse syncinfo output of the given job_id for the "Job state"
    # field and return the value
    
    my $job_id = shift;
    
    dbgprint "GetJobStatus for job_id $job_id\n";

    my $syncinfo = DoCmdNoTranslate("$CFG::MultiTool syncinfo -job $job_id");

    if ($? != 0) {
        dbgprint "Could not get information for job: $job_id.\n";
        return undef;
    }

    my @lines = split("\n", $syncinfo);
    my $job_state = '';

    foreach my $line (@lines) {
        if ($line =~ /\s+Job state:\s+(.*)$/) {
            $job_state = $1;
            dbgprint "Job state for job $job_id: $job_state.\n";
            return $job_state;
        }
    }
    return undef;
}

sub GetLspacketReps {
    # Parse the output from 'lspacket -long' to generate a list of local vobs
    # that this packet may be applied to.
    # input:
    #   packet path
    # Returns:
    # number of local replicas this packet applies to, and the list of vobtags

    my $pkt_name = shift;

    dbgprint "GetLspacketReps: for pkt: $pkt_name\n";

    if (!(-f $pkt_name)) {    # reject if directory or non existent
        return (0, undef);
    }

    my $pkt_spec = qq/"$pkt_name"/;

    my @vobtag_list = ();
    my $vob_tag = "";

    my $lspacket = DoCmdNoTranslate("$CFG::MultiTool lspacket -long $pkt_spec");

    if ($? != 0) {
        dbgprint "$pkt_name is not a packet, ignoring.\n";
        return (0, undef);
    }

    my @lines = split("\n", $lspacket);

    my $checkline = 0;
    my $pkt_path = '';
    my $num_loc_replicas = 0;

    foreach my $line (@lines) {
        # collect replica lines after 'packet intended' message; turn off
        # collection after hitting 'originating replica...'
        #
        # parse the packet path name
        if ($line =~ /^Packet is:\s+(.*)$/) {
            $pkt_path = $1;
        }

        if ($line =~ /Packet type: Replica Creation/) {
            # ignore a replica creation packet!
            dbgprint "packet is a replica creation packet; ignoring it.\n";
            next;
        }

        if ($line =~ /^Packet intended for the following targets/) {
            $checkline = 1;
            next;
        } elsif ($line =~ /^Originating replica is/) {
            $checkline = 0;
            next;
        } elsif ($checkline) {
            # found a replica line, parse out repl. name and tag values, and
            # add to the list. Don't match lines unless have "[ local to..."
            if ($line =~ m/\s+([\S]+)\s+\[.*\]\s+[\*\s]*tag:\s+(.+)$/) {
                dbgprint "local replica found: $1 tag: $2\n";
                if ($1 && $2) {
                    $vobtag_list[$num_loc_replicas++] = $2;
                }
            } else {
                # replica found, but not on this host (no "[ local to... ]")
                # so ignore it.
            }
        }
    }
    return ($num_loc_replicas, @vobtag_list);
}

sub ImportFile {
    # figure out each file we found: is it a packet? is it applicable?
    # if "-vob_pattern" spec'd, can packet apply to all local replicas?
    # IN  file name of potential packet to import

    my $pkt = shift;

    dbgprint "examining file: $pkt\n";
    
    my $tmpfile;
    my $file_was_compressed = 0;

    my $name_gz = $pkt;
	
    # check if filename matches ".gz",  ".gz_1", ".GZ9", etc.; if so, try
    # decompressing it. If it fails, attempt to process as a regular
    # file.
    if ($pkt =~ m/(.*)\.(gz|gz(\d*)(_{0,1})(\d*))$/i) {     # compressed file
        my $name = $1;
        my $ext = $2;
        my $filedir = dirname($pkt);   # from File::Basename module
	my $success;

        dbgprint "probable compressed file found: $pkt\n";
        $tmpfile = ConsTmpFile($filedir . $CFG::FS . "tpkt");
	($name_gz, $success) = RenameGZFileName($name, $ext);
	if ($success == 0) {
	    # We can't trust the new filename
	    die "Unable to rename packet $datafile to $name_gz: $!\n";		      }
        $res = Do_System("$CFG::compress_pgm -c -d \"$name_gz\" > \"$tmpfile\"",
                         __FILE__, __LINE__);
        if ($res == 0) {
            $file_was_compressed = 1;
        } else {
            # some error decompressing the file, try to process as uncompressed
            print2 "INFO: '$pkt' could not be decompressed!\n" .
                "Attempting to process as an uncompressed file.\n";
            unlink $tmpfile;
            $tmpfile = $pkt;
        }
    } else {
        $tmpfile = $pkt;
    }

    # use 'lspacket' to filter out non-packets
    $res = Do_System("$CFG::MultiTool lspacket \"$tmpfile\" > $CFG::nulldevice",
                     __FILE__, __LINE__); 
   if ($res != 0) {
        # if not a packet, remove the tmp uncompressed file we just created,
        # and leave the original for another application to process
        dbgprint "file is not recognized as a packet, skipping: $tmpfile\n";
        if ($file_was_compressed) {
            unlink $tmpfile;
        }
        next;
    }

    # if we decompressed a real packet, then remove the compressed version.
    if ($file_was_compressed) {
        unlink $name_gz;
    }

    # if a "vob pattern" was specified, figure out if we can apply this packet
    # to all, or any, local replicas

    my @invob_list = ();
    my $use_invob_parm = 0;
    my $num_requests = 1;    # number of "sync -import" requests to make

    dbgprint "ImportPkt(): vob_pattern: $vob_pattern\n\n";
    if ($vob_pattern) {
        my $reps_in_pkt;
        ($reps_in_pkt, @invob_list) = GetLspacketReps($tmpfile);
        dbgprint "GetLspacketReps: vobs in pkt: $reps_in_pkt @invob_list\n";

        @invob_list = grep /$vob_pattern/, @invob_list;

        if ($reps_in_pkt != (scalar @invob_list)) {  # all reps match pattern?
            # no - do request for each matching vob individually, with "-invob"
            $num_requests = scalar @invob_list;
            $use_invob_parm = 1;
        }
    }

    my $other_expr = $ENV{MULTISITE_IMPORT_OTHER_PARAMS};

    for (my $j = 0; $j < $num_requests; $j++) {
        my $invobexpr = "";
        if ($use_invob_parm) {
            $invobexpr = "-invob " . $invob_list[$j];
        }
        $res = Do_System("$multitool_cmd $other_expr $invobexpr \"$tmpfile\"", 
                         __FILE__, __LINE__);
        if ($res !=0) {
            # syncreplica returning a fail status does not necessarily mean
            # that the sync job failed.To really find out if the job failed, 
            # parse job_id_tmpfile for job ids and check the Job State
            my @job_list = GetJobIds($job_id_tmpfile);
            if ((scalar @job_list) > 0) {
                foreach my $jobid (@job_list) {
                    my $job_state = GetJobState($jobid);
                    if ($job_state =~ /failed/) {
                        # The sync job failed.
                        # Update the event log w/pointer to the log
                        if ($CFG::NT) {
                            $cmd = "$CFG::imsglog sync_import evtlog \"$logname\"";
                            $res = Do_System($cmd, __FILE__, __LINE__);
                        }
                        print STDERR "INFO: MultiSite import job $jobid failed.\n";
                        dbgprint "INFO: import failed." .
                        " See logfile in \"$logname\"\n";
                        $CFG::failcount++;
                    }
                }
            } else {
                # The sync failed. No job created
                # Update the event log w/pointer to the log
                if ($CFG::NT) {
                    $cmd = "$CFG::imsglog sync_import evtlog \"$logname\"";
                    $res = Do_System($cmd, __FILE__, __LINE__);
                }
                print STDERR "INFO: MultiSite import failed.\n";
                dbgprint "INFO: import failed." .
                    " See logfile in \"$logname\"\n";
                $CFG::failcount++;
            }
        } else { # ($res == 0)
            # successful import
            if ($CFG::NT) {
                $cmd = "$CFG::imsglog sync_import evtlog";
                $res = Do_System($cmd, __FILE__, __LINE__);
            }
            
            # if duration >= 0, blocking or wait_forever were specified.
            if ($nblocking) {
                print STDERR "MultiSite import request successfully queued.\n";
                } else {
                    print STDERR "MultiSite import successfully processed.\n";
                }
        }
        unlink $job_id_tmpfile;
    }
}

sub GetAllBays {
    my @bays;
    my $storageclass = shift;
    if ($CFG::NT) {
        my $hkey = Win32::TieRegistry->new("LMachine\\$RGY::ms_st_regy\\",
                   {Access=>'KEY_READ'});
        dbgprint "hkey: $hkey\n";
        if($hkey) {
            my @sclasses = ($storageclass ne '') ? 
                ($storageclass) : $hkey->SubKeyNames();
            foreach my $class (@sclasses) {
                dbgprint "foreach storage class $class\n";
                my $bay = GetBaysFromSclass($class);  # only one bay/class on NT
                push(@bays, $bay) if($bay ne '');
                undef $hkey;
            }
        }
    } else {
        my $cmd = qq@grep STORAGE-BAY $CFG::ship_conf | grep -v "#" @;
        my $baylist = DoCmd($cmd);
        my @bay_lines = split("\n", $baylist);

        foreach my $bay_info (@bay_lines) {
            my ($sb, $class, $bay_path) = split(/\s+/, $bay_info, 3);
            dbgprint "$class path: $bay_path\n";
            if (!$sclass || $sclass eq $class) {
                push(@bays, $bay_path);
            }
        }
    }
    dbgprint "for storage class '" . ($sclass ? $sclass : "(all classes)") . 
        "' found bays: " . join (" ", @bays) . "\n";
    return @bays;
}

sub GetBaysFromSclass {
    # return bays for a given shipping class (NT only!)
    my $sclass = shift;

    dbgprint "GetBaysFromSclass ($sclass)\n";
    my $result;
    my $keyname = "LMachine\\$RGY::ms_st_regy\\$sclass";
    dbgprint "looking up registry key: $keyname\n";
    my $key = Win32::TieRegistry->new($keyname, {Access=>'KEY_READ'});
    dbgprint "key found is: $key\n";
    if (!defined $key) {
        qprint "ERROR: could not get registry key $keyname!\n";
        undef $result;
    } else {
        my $value = $key->GetValue("StorageBay");
        dbgprint "StorageBay value is: $value\n";
        if($value eq '') {
            qprint "WARNING: Unable to get storage bay for class $sclass\n";
        } else {
            $result = $value;
        }
        undef $key;
    }

    return $result;
}

sub GetDirFiles {
    # get list of all files in "$dir"
    my $dir = shift;
    my @files = ();

    if (opendir(THEDIR, $dir)) {
        @files = grep {-f} map {"$dir${CFG::FS}$_"} readdir THEDIR;
        closedir THEDIR;
    }
    return @files;
}

sub GetArchitecture {
    # figure out architecture we're on. Config.pm package and $^O Perl variable
    # both have changed depending on the version of perl or ccperl that actually
    # runs the script, so we need to try a couple of combinations.

    # This should go in syncmgr_common.pl, EXCEPT that we need to figure out the
    # architecture and know the PATH to syncmgr_common.pl before we can 
    # 'require' it.

    my $arch = $^O ? $^O : $main::Config{archname};

    my $isNT = (($arch =~ /^MSWin32/) || ($arch eq 'i386-win32')) ? 1 : 0;
    my $isUNIX = ($CFG::NT ? 0 : 1);

    return ($isNT, $isUNIX);
}

__END__
:endofperl
ccperl -S ccperlpp.pl %0.bat %*
